/*******************************************************************************
                                    ReSyEqVa                                    
                                    ========                                    

Resolve Systems of 2 Equations in 2 Variables
Copyright (C) 2010 Paul Kocialkowski

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

*******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <stdbool.h>

#include "fract.h"

int gcf(int gcf_1, int gcf_2)
{
	int gcf_result=0;
	int i;

	while(!gcf_result)
	{
		i=gcf_1 % gcf_2;
	
		if(i == 0)
			gcf_result=gcf_2;
		else
		{
			gcf_1=gcf_2;
			gcf_2=i;
		}
	}

	return gcf_result;
}

int lcm(int lcm_1, int lcm_2)
{
	/* Compute LCM from GCF */
	int lcm_result=(lcm_1 * lcm_2) / gcf(lcm_1, lcm_2);

	return lcm_result;
}

fract fract_red(fract fract_1)
{
	int fract_gcf=gcf(fract_1.n, fract_1.d);

	fract fract_result;
	fract_result.n=fract_1.n / fract_gcf;
	fract_result.d=fract_1.d / fract_gcf;

	return fract_result;
}


fract fract_add(fract fract_1, fract fract_2)
{
	int fract_lcm=lcm(fract_1.d, fract_2.d);

	fract fract_result;

	fract_1.n=fract_1.n * (fract_lcm / fract_1.d);
	fract_2.n=fract_2.n * (fract_lcm / fract_2.d);

	fract_result.n=fract_1.n + fract_2.n;
	fract_result.d=fract_lcm;

	fract_result=fract_red(fract_result);

	return fract_result;
}

fract fract_del(fract fract_1, fract fract_2)
{
	int fract_lcm=lcm(fract_1.d, fract_2.d);

	fract fract_result;

	fract_1.n=fract_1.n * (fract_lcm / fract_1.d);
	fract_2.n=fract_2.n * (fract_lcm / fract_2.d);

	fract_result.n=fract_1.n - fract_2.n;
	fract_result.d=fract_lcm;

	fract_result=fract_red(fract_result);

	return fract_result;
}

fract fract_mul(fract fract_1, fract fract_2)
{
	int fract_gcf;
	bool less_than_zero=false;

	fract fract_result;

	fract_result.n=fract_1.n * fract_2.n;
	fract_result.d=fract_1.d * fract_2.d;

	if(fract_result.n < 0)
	{
		less_than_zero=true;
		fract_result.n=fract_result.n * -1;
	}
	if(fract_result.d < 0)
	{
		if(less_than_zero)
			less_than_zero=false;
		else
			less_than_zero=true;

		fract_result.d=fract_result.d * -1;
	}
		
	fract_result=fract_red(fract_result);

	if(less_than_zero)
		fract_result.n=fract_result.n * -1;

	return fract_result;
}

fract fract_from_num(int fract_n)
{
	fract fract_result;

	fract_result.n=fract_n;
	fract_result.d=1;

	return fract_result;
}

fract fract_from_den(int fract_d)
{
	fract fract_result;

	fract_result.n=1;
	fract_result.d=fract_d;

	return fract_result;
}
